/**
 *  @author : gaoyong
 *  @data : 2016.05.23
 */
package org.fenode.modle;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;

public class MyRequest {
	private String requestURI;
	private String requestURL;
	private LinkedHashMap<String, String> headers;
	private ArrayList<Cookie> cookies;
	private String queryString;
	private LinkedHashMap<String, String> parameterMap;
	private String serverName;
	private String scheme;
	private String protocol;
	private int serverPort;

	public MyRequest(Map<String, Object> data) throws JsonParseException,
			JsonMappingException, IOException {
		
		headers = (LinkedHashMap<String, String>) data.get("headers");
		
		String cookieString = headers.get("Cookie");
		if (cookieString != null && !"".equals(cookieString)) {
			cookies = new ArrayList<Cookie>();
			String[] cc = cookieString.split("; *");
			for (int i = 0; i < cc.length; i++) {
				String[] oneCookie = cc[i].split("=");
				cookies.add(new Cookie(oneCookie[0], oneCookie[1]));
			}
		}
		parameterMap = (LinkedHashMap<String, String>) data.get("query");
		if (parameterMap != null && parameterMap.size() > 0) {
			StringBuilder sb = new StringBuilder();
			for (String key : parameterMap.keySet()) {
				sb.append(key);
				sb.append("=");
				sb.append(parameterMap.get(key));
				sb.append("&");
			}
			queryString = sb.toString();
		}
		if ("".equals(queryString) || queryString == null) {
			queryString = null;
		} else {
			queryString = queryString.substring(0, queryString.length() - 1);
		}
		serverName = (String) data.get("hostname");
		String pathname = (String) data.get("pathname");
		scheme = (String) data.get("protocol");
		String httpVersion = (String) data.get("httpVersion");
		protocol = scheme + "/" + httpVersion;
		requestURI = pathname;
		requestURL = scheme + "://" + serverName + pathname;
		serverPort = (Integer) data.get("port");
	}

	public String getRequestURI() {
		return requestURI;
	}

	public String getRequestURL() {
		return requestURL;
	}

	public ArrayList<Cookie> getCookies() {
		return cookies;
	}

	public Set<String> getHeaderNames() {
		return headers.keySet();
	}

	public String getHeader(String key) {
		return headers.get(key);
	}

	public String getQueryString() {
		return queryString;
	}

	public String getParameter(String key) {
		return parameterMap.get(key);
	}

	public LinkedHashMap<String, String> getParameterMap() {
		return parameterMap;
	}

	public Set<String> getParameterNames() {
		return parameterMap.keySet();
	}

	public String getServerName() {
		return serverName;
	}

	public int getServerPort() {
		return serverPort;
	}

	public String getProtocol() {
		return protocol;
	}
	public String getScheme() {
		return scheme;
	}
}
