/**
 * @author : gaoyong
 * @data : 2016.05.23
 */
package org.fenode.parse;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.fenode.modle.MyRequest;

public abstract class TemplateEngineParser {
	/**
	 *  路径map
	 *  dirpath : ""  模版引擎文件所在文件夹的绝对路径 eg: "/home/~/"
	 *  filename : ""  模版引擎文件文件名,带后缀  eg: "index.ftl" "index.vm"
	 */
	protected HashMap<String,String> templateFileInfo = new HashMap<String,String>();
	
	/**
	 * http 请求头中包含的信息
	 */
	protected LinkedHashMap<String, Object> httpInfo = new LinkedHashMap<String,Object>();;
	
	/**
	 * 解析json字符串中包含的解析信息
	 * @param fileInfoJSON  文件信息
	 * @param httpInfoJSON  http头信息
	 * @throws JsonParseException  
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	protected void parseInfo (final String fileInfoJSON,final String httpInfoJSON) throws JsonParseException, JsonMappingException, IOException,Exception {
		if ( null == fileInfoJSON || null == httpInfoJSON ) {
			throw new Exception("parseInfo cannot be null!");
		}
		
		ObjectMapper mapper = new ObjectMapper();
		mapper.configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true);
		
		LinkedHashMap<String, Object> tmpFileInfo = (LinkedHashMap<String, Object>) mapper.readValue(fileInfoJSON,Map.class);
		String dirpath = (String) tmpFileInfo.get("dirpath");
		String filename = (String) tmpFileInfo.get("filename");
		this.templateFileInfo.put("dirpath", dirpath);
		this.templateFileInfo.put("filename", filename);
		
		this.httpInfo = (LinkedHashMap<String,Object>) mapper.readValue(httpInfoJSON, Map.class);
		
	}
	
	/**
	 * 将http头中的信息封装成对象
	 * @return
	 * @throws JsonParseException
	 * @throws JsonMappingException
	 * @throws IOException
	 */
	protected HashMap<String,Object> mockData () throws JsonParseException, JsonMappingException, IOException {
		HashMap<String, Object> mockData = new HashMap<String, Object>();
		if (null != this.httpInfo && this.httpInfo.size() > 0) {
			for(String key: this.httpInfo.keySet()) {
				if (key.equals("request")) {
					 Map<String, Object> req = (Map<String, Object>) this.httpInfo.get("request");
					 MyRequest request = new MyRequest(req);		
					 LinkedHashMap<String, String> parameters = request.getParameterMap();
					 mockData.put("Parameters", parameters);
					 mockData.put("request", request);
					 LinkedHashMap<String, String> body = (LinkedHashMap<String, String>)req.get("body");
					 if (body != null && body.size() > 0) {
						 for(String k: body.keySet()) {
							 mockData.put(k, body.get(k));
						 }
					 }	
				}
			}
		}
		
		return mockData;
	}
}

