'use strict'
/**
 * Created by gaoyong on 16/6/29.
 */
const fs = require('fs');
const http = require('http');
const path = require('path');
const child_process = require('child_process');

const koa = require('koa');
const app = koa();
const bodyParser = require('koa-bodyparser');
const serve = require('koa-static');
const render = require('koa-ejs');
const route = require('koa-router')();
const cookieParser = require('cookie-parser');

const appManager = require('../util/appManager.js');
const http404PagePath = path.join(__dirname,'../../config/404.html');

const webService = require('./webService.js');

function init ( webPort ) {
	// 内部使用静态文件加载
	app.use(serve(path.join(__dirname, '../../web/static/')));
	// 设置模版
	render(app,{
		root: path.join(__dirname,'../../web/views/'),
		layout: false,
		viewExt: 'ejs',
		cache: false,
		debug: true
	});

	// body parser for koajs
	app.use(bodyParser());
	
	webService( app,route,webPort );

	app.use( route.routes() )
        .use( route.allowedMethods() );



	// 404
    route.post('/**',function* () {
        this.body = yield Promise.resolve({ 'http state code' : '404' });
    });
    route.get('/**',function* () {
        this.body = yield (new Promise( function (resolve,reject) {
            fs.readFile(http404PagePath,'utf8',function (err,data) {
                if ( err ) {
                    reject('<strong>404</strong>>');
                }
                resolve(data);
            });
        } ));
    });

	app.listen(webPort);
	// 打开站点
	openWebsite(webPort);
}

function openWebsite (webPort) {
	console.log('服务器成功启动', '端口号码', webPort);
	// 启动一个默认浏览器打开后台管理页面
	let cmd, uri = "http://127.0.0.1" + (webPort == 80 ? "" : ":" + webPort);
	uri += "/";
	if (process.platform === 'win32') {
		cmd = 'start';
	} else if (process.platform === 'linux') {
		cmd = 'xdg-open';
	} else if (process.platform === 'darwin') {
		cmd = 'open';
	}
	console.log('fn-server服务器后台管理页面打开中...');

	child_process.exec([cmd, uri].join(' '));

	console.log(`\nfn-server manager uri: ${uri} \n`);
}

module.exports = {
	init: function (webPort) {
		init( webPort );
	}
};

