'use strict'
/**
*  系统服务
*  @date: 2016.07.05
*/
const fs = require('fs');
const path = require('path');
const url = require('url');
const querystring = require('querystring');

const parse = require('co-body');

const appManager = require('../util/appManager.js');


/**
*  得到系统配置
*/
function getSystemSetting () {
	return new Promise( function (resolve,reject) {
		try {
			_getSystemSetting(resolve,reject);
		} catch (e) {
			resolve( {
				'err_code': -1,
				'msg': '读取系统配置时发生错误!'
			} );
		}
	} );
}
function _getSystemSetting (resolve,reject) {
	let config = appManager.getSystemConfig();
	if ( config ) {
		config.configHttpPath = encodeURIComponent( config.configHttpPath );
		resolve( config );
	} else {
		resolve( {
			'err_code': -1,
			'msg': '读取系统配置时发生错误!'
		} );
	}
}
/**
*  保存系统配置
*/
function saveSystemSetting (context) {
	return new Promise( function (resolve,reject) {
		try {
			_saveSystemSetting(context,resolve,reject);
		} catch (e) {
			resolve( {
				'err_code': -1,
				'msg': '读取系统配置时发生错误!'
			} );
		}
	} );
}
function _saveSystemSetting (context,resolve,reject) {
	let _url = context.req.url;
	let query = url.parse(_url).query;
	let paramObj = querystring.parse(query);
	paramObj.configHttpPath = decodeURIComponent(paramObj.configHttpPath);

	let appStartupObj = {
		'port': parseInt(paramObj.servicePort),
		'webPort': parseInt(paramObj.webPort),
		'configHttpPath': paramObj.configHttpPath,
		'domain': paramObj.httpDomain
	};

	let ret = appManager.saveSystemConfig( appStartupObj );
	if ( ret === true ) {
		resolve( {
			'err_code': 0
		} );
	} else {
		resolve( {
			'err_code': -1,
			'msg': '系统配置保存错误,' + ret
		} );
	}
}


module.exports = {
	getSystemSetting: getSystemSetting,
	saveSystemSetting: saveSystemSetting
};

