'use strict'
/**
*   模版引擎落地页服务
*/
const fs = require('fs');
const path = require('path');
const url = require('url');

const Render = require('../service/renders/Render.js');
const httpUtil = require('../util/httpUtil.js');
const fileUtil = require('../util/fileUtil.js');
const appManager = require('../util/appManager.js');


function getArtTemplateRoutes () {
	return new Promise( (resolve,reject) => {
		try {
			_getArtTemplateRoutes(resolve,reject);
		} catch (e) {
			resolve(e);
		}
	} );
}
function _getArtTemplateRoutes (resolve,reject) {
	const port = appManager.getServicePort();

	httpUtil.reloadLandPages().then( function (routes) {
		if ( !routes || routes.length <= 0 ) {
			resolve({
				'err_code': -1,
				'msg': '没有配置artTemplate工程!'
			});
			return false;
		}

		const artTemplatList = [];
		for ( let route of routes ) {
			/**
			*  判断配置是否合法并且渲染文件是否存在
			*/
			if ( Render.isValidateArtTemplate(route) &&
						fileUtil.fileExists( route.filePath ) ) {
				artTemplatList.push( route );
			}
		}

		if ( artTemplatList.length > 0 ) {
			let projects = [],
					projectsObj = {};
			for ( let artTemplat of artTemplatList ) {
				let pName = artTemplat.projectName || 'anonymous';
				if ( !(pName in projectsObj) ) {
					projectsObj[pName] = {
						projectName: pName,
						routes: []
					};
				}

				artTemplat.fnsURI = httpUtil.addPort2URI( artTemplat.uri );
				if ( !artTemplat.fnsURI ) { // 跳过不合法的uri配置
					continue;
				}

				projectsObj[pName].routes.push( artTemplat );
			}

			Object.keys(projectsObj).forEach( function (key) {
				projects.push( projectsObj[''+key] );
			} );

			resolve( {
				'err_code': 0,
				'artTemplatList': projects
			} );
		} else {
			resolve({
				'err_code': -1,
				'msg': '没有配置artTemplate工程!'
			});
		}
	} ).catch( function (err) {
		resolve({
			'err_code': -1,
			'msg': err
		});
	} );
}





module.exports = {
	getArtTemplateRoutes: getArtTemplateRoutes
};


