'use strict'
/**
*   fn-server配置服务
*/
const fs = require('fs');
const path = require('path');
const url = require('url');
const querystring = require('querystring');

const parse = require('co-body');

const appManager = require('../util/appManager.js');
const systemService = require('./systemService.js');
const templateService = require('./templateService.js');

/**
*  得到http配置path
*/
function getFNSHttpPath () {
	return new Promise( function (resolve,reject) {
		try {
			let httpPath = appManager.getHttpPath();
			resolve( httpPath );
		} catch (e) {
			resolve('服务器获取http配置路径报错!');
		}
	} );
}

/**
*  保存http配置path
*  @param: context -- Koajs 上下文对象
*/
function saveFNSHttpPath (context) {
	return new Promise( function (resolve,reject) {
		try {

			let originalUrl = context.request.originalUrl;
	        let urlObj = url.parse( originalUrl );    
	        let queryObj = querystring.parse( urlObj.query );
	        let httpPath = queryObj['httpPath'];
	        httpPath = decodeURIComponent( httpPath );
	        if ( path.isAbsolute( httpPath ) ) {
	        	let ret = appManager.saveHttpPath( httpPath );
	        	if ( ret === true ) {
	        		resolve( { 'err_code':0 } );
	        	} else {
	        		resolve(ret);
	        	}
	        } else {
				resolve('服务器保存httpPath配置是非绝对路径!');
	        }
		} catch (e) {
			resolve('服务器保存http配置路径报错!');
		}
	} );
}

function getFNSHttps () {

}

function init (app,route,webPort) {
	let _cdnBaseUrl = `http://localhost:${webPort}`;

	// 首页
	route.get('/', function* () {
		yield this.render('overview',{
			'cdnBaseUrl': _cdnBaseUrl
		});
	});

	// 系统配置页
	route.get('/setting', function* () {
		yield this.render('setting',{
			'cdnBaseUrl': _cdnBaseUrl
		});
	});
	route.get('/getSystemSetting', function* () {
		this.body = yield systemService.getSystemSetting();
	});
	route.get('/saveSystemSetting',function* () {
		this.body = yield systemService.saveSystemSetting(this);
	});

	// artTemplate路由页
	route.get('/artTemplate',function* () {
		yield this.render('artTemplate',{
			'cdnBaseUrl': _cdnBaseUrl
		});
	});
	route.get('/getArtTemplateRoutes',function* () {
		this.body = yield templateService.getArtTemplateRoutes();
	});


	
};

module.exports = function (app,route,webPort) {
	if ( app && typeof app === 'object' ) {
		init( app,route,webPort );
	}
};

