'use strict'
/**
 * Created by gaoyong on 16/6/28.
 */
const fs = require('fs');

const glob = require('glob');

const appManager = require('../util/appManager.js');
const httpUtil = require('../util/httpUtil.js');

class AjaxService {
    constructor () {

    }

    /**
    *  返回ajax请求结果 Promise对象
    */
    handle (context,httpRequst) {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._handle(context,httpRequst,resolve,reject);
            } catch (e) {
                console.log(`ajax not match.`);
                reject(e);
            }
        } );
    }

    _handle ( context,httpRequst,resolve,reject ) {
        const _this = this;

        this.findMatchAjaxByRequst(httpRequst).then( function (ajax) {
            if ( ajax && typeof ajax === 'object' ) {
                // ajax 响应等待时间
                let _timeout = ajax.timeout > 0 ? ajax.timeout: 0;
                _timeout = parseInt( _timeout );
                // 响应结果
                let json = {
                    'tip': '忘记配置ajax请求中的json字段了.'
                };
                if ( ajax.json && typeof ajax.json === 'object' ) {
                    json = ajax.json;
                }

                // 等待_timeout毫秒后将结果返回给页面
                setTimeout( function () {
                    context.body = json; // koa结果赋值
                    resolve( true );
                },_timeout );
            } else {
                resolve(false); //未找到
            }
        } ).catch( function (e) {
            console.log(`ajaxService _handle error: ${e}.`);
        } );
    }


    /**
     *  根据httpRequst 找出匹配的第一个ajax请求配置信息
     *   @return ajaxObj
     *      如果找不到匹配的配置信息的话 则返回值中eg null
     */
    findMatchAjaxByRequst ( httpRequst ) {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._findMatchAjaxByRequst(httpRequst,resolve,reject);
            } catch (e) {
                reject(`ajaxService findMatchAjaxByRequst method error.`);
            }
        } );
    }
    _findMatchAjaxByRequst ( httpRequst,resolve,reject ) {
        // 得到http ajax请求的uri path部分 和 http请求类型
        const uriPath = httpRequst.path;
        const httpMethod = httpRequst.method.toLowerCase();

        /**
         *   找到ajax请求路径对应的配置信息
        */
        httpUtil.reloadAjaxDatas().then( function (ajaxs) {
            if ( ajaxs && Array.isArray(ajaxs) && ajaxs.length > 0 ) {
                for ( let ajax of ajaxs ) {
                    ajax.method = ajax.method.toLowerCase();
                    if ( httpUtil.isValidAjaxSetting(ajax) ) {
                        if ( ajax.uri === uriPath && 
                                    ajax.method === httpMethod ) {
                            // 找到了匹配ajax信息
                            resolve( ajax );

                            return true;// 终止函数
                        }
                    }
                }

                resolve( null );  //找不到对应的ajax配置
            } else {
                resolve( null );  //找不到对应的ajax配置
            }
        } ).catch( function (e) {
            console.log(`ajaxService _findMatchAjaxByRequst error: ${error}.`);
            reject( error );
        } );
    }
}


const ajaxService = new AjaxService();

module.exports = function (context,httpRequst) {
    return ajaxService.handle(context,httpRequst);
};
