'use strict'
/**
 * Created by gaoyong on 16/5/26.
 */
const path = require('path');

const httpUtil = require('../util/httpUtil.js');

// 渲染类
const Render = require('./renders/Render.js');
const HtmlRender = require('./renders/HtmlRender');
const FreemarkerRender = require('./renders/FreemarkerRender.js');
const VelocityRender = require('./renders/VelocityRender.js');

class PageRender {
    constructor () {
        this.renders = {};
        this.renders['html'] = new HtmlRender();
        this.renders['ftl'] = new FreemarkerRender();
        this.renders['vm'] = new VelocityRender();
    }

    handle (context,httpRequst) {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._handle( context,httpRequst,resolve,reject );
            } catch (e) {
                console.log(`pageRender handle error: ${e}.`);
            }
        } );
    }

    _handle (context,httpRequst,resolve,reject) {
       //  根据httpRequst调用相应的render解析文件
       this.findRenderByRequst( httpRequst ).then( function (renderInfo) {
            if ( !renderInfo || !renderInfo[0] || !renderInfo[1] ) {
               resolve( false );
               return false;
            }

            const render =  renderInfo[0],
                  filePath =  renderInfo[1],
                   renderData = renderInfo[2];
            render.render( filePath,httpRequst,renderData ).then( function ( data ) {
               context.body = data;

               resolve( true );
            } ).catch( function (e) {
               resolve( false );
            } );
       } ).catch( function (error) {
            reject( error );
       } );
    }

    /**
     *  根据httpRequst 算出需要调用的解析器和渲染文件的绝对路径
     *   @return [ {},'/~/a.ftl' ]
     *      如果渲染文件不支持或者找不到渲染文件的话 则返回值中eg [ null,'' ]
     */
    findRenderByRequst ( httpRequst ) {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._findRenderByRequst(httpRequst,resolve,reject);
            } catch (e) {
                reject(`pageRender _findRenderByRequst method error.`);
            }
        } );
    }
    _findRenderByRequst ( httpRequst,resolve,reject ) {
        const _this = this;

        // 得到http request的 path
        const httpURI = httpRequst.uri;

        /**
         *   算出虚拟路径对应的渲染器
        */
        httpUtil.reloadLandPages().then( function (routes) {
            if ( routes && Array.isArray(routes) ) {
                let route = null;
                let localVirtualPath = '',
                            localFilePath = '';
                
                // 遍历所有路由集合
                for ( let route of routes ) {
                    let filePath = route['filePath'];
                    //let uriPath = httpUtil.a( route['uri'] );
                    if ( httpURI === route['uri'] ) {
                        if ( Render.validateFileType(route.type) && 
                                    Render.validateFileSuffix(filePath) ) {
                            if (!path.isAbsolute(filePath)) {
                                continue;
                            }
                            let data = {};
                            if ( Render.isValidateArtTemplate(route) ) { //artTemplate模版
                                Object.keys(route.json).forEach( function (key) {
                                    if ( typeof key != 'function' ) {
                                        data[''+key] = JSON.stringify( route.json[''+key] );
                                    }
                                } );
                            }

                            let fileRender = _this.getFileRenderByFilename(filePath);
                            if (fileRender) {
                                resolve( [fileRender,filePath,data] );

                                return true;// 终止函数
                            }
                        }
                    }
                }

                resolve( [ null,'',null ] );  //找不到渲染对应的路由
            } else {
                reject( [ null,'' ] );
            }
        } ).catch( function (error) {
            console.log(`PageRender _render error: ${error}.`);
            reject( error );
        } );
    }



    /**
     * 根据渲染文件绝对路径得到渲染器
     * @param filepath
     * @returns {*}
     */
    getFileRenderByFilename ( filepath ) {
        if ( filepath.endsWith( '.html' ) ) {
            return this.renders['html'];
        } else if ( filepath.endsWith( '.ftl' ) ) {
            return this.renders['ftl'];
        } else if ( filepath.endsWith( '.vm' ) ) {
            return this.renders['vm'];
        }

        return null;
    }


}


const pageRender = new PageRender();

let visitFrequency = 1;
let cursor = 0;
module.exports = function (context,httpRequst) {
    if ( visitFrequency == 0 ) {
        return new Promise( function ( resolve,reject ) {
            resolve( false );
        } );
    }
    visitFrequency = 0;
    setTimeout( function () {
        visitFrequency = 1;
    } ,30);

    return pageRender.handle(context,httpRequst);
};
