'use strict'
/**
 * Created by gaoyong on 16/5/30.
 */
const path = require('path');

const regUtil = require('../../util/regUtil.js');

// 合法的渲染文件后缀列表
const renderFileSuffix = ['.ftl','.vm','.html'];
const renderFileType = ['artTemplate','freemarker','velocity'];

class Render {
    constructor () {

    }

    /**
     * 检查文件是否为合法渲染文件类型
     *  @param file : eg a.ftl
     *  @return : true/false
     */
    static validateFileType ( fileType ) {
        if ( !fileType || typeof fileType != 'string' ) {
            return false;
        }

        for (let type of renderFileType) {
            if ( type === fileType ) {
                return true;
            }
        }

        return false;
    }

    /**
     * 检查文件是否为合法渲染文件后缀
     *  @param file : eg a.ftl
     *  @return : true/false
     */
    static validateFileSuffix ( filePath ) {
        if ( !filePath || typeof filePath != 'string' ) {
            return false;
        }

        for ( let suffix of renderFileSuffix ) {
            if ( filePath.endsWith( suffix ) ) {
                return true;
            }
        }

        return false;
    }

    /**
    *  是否是合法的artTemplate配置
    *   @route -- fn-server服务器 route配置项
    */
    static isValidateArtTemplate ( route ) {
        if ( !route || typeof route != 'object' ) {
            return false;
        }

        if (  route.uri && regUtil.validateURI(route.uri) &&  
                route.type === 'artTemplate' && 
                    typeof route.filePath === 'string' &&
                        route.filePath.endsWith('.html') ) {
            return true;
        }

        return false;
    }

    /**
    *  是否是合法的Freemarker配置
    *   @route -- fn-server服务器 route配置项
    */
    static isValidateFreemarker ( route ) {
        if ( !route || typeof route != 'object' ) {
            return false;
        }

        if ( route.uri && regUtil.validateURI(route.uri) && 
                route.type === 'freemarker' && 
                  typeof route.filePath === 'string' &&
                            route.filePath.endsWith('.ftl') ) {
            return true;
        }

        return false;
    }

    /**
    *  是否是合法的Velocity配置
    *   @route -- fn-server服务器 route配置项
    */
    static isValidateVelocity ( route ) {
        if ( !route || typeof route != 'object' ) {
            return false;
        }

        if ( route.uri && regUtil.validateURI(route.uri) && 
                route.type === 'velocity' && 
                    typeof route.filePath === 'string' &&    
                        route.filePath.endsWith('.vm') ) {
            return true;
        }

        return false;
    }

    /**
    *  是否是合法的渲染页 route配置项
    *   @param route -- route配置项
    */
    static isValidatePageSetting ( route ) {
        if ( Render.isValidateArtTemplate(route) ||
                Render.isValidateFreemarker(route) ||
                    Render.isValidateVelocity(route) ) {
            return true;
        }

        return false;
    }

    /**
    *   是否是合法的ajax route配置项
    *    @param route -- route配置项
    */
    static isValidAjaxSetting ( route ) {
        if ( !route || typeof route != 'object' ) {
            return false;
        }

        if ( route.uri && route.uri.startsWith('/') && 
                (route.type === 'get' || route.type === 'post') &&
                        route.json && typeof route.json === 'object' ) {
            return true;
        }

        return false;
    }

}

module.exports = Render;
