'use strict'
/**
 * Created by gaoyong on 16/5/31.
 */
const spawn = require('child_process').spawn;
const fs = require('fs');
const path = require('path');

const Render = require('./Render.js');
const jarFilePath = path.join( __dirname,'../../../lib/java-template/template.jar' );

class VelocityRender extends Render {
    constructor () {
        super( arguments );
    }

    /**
     * Velocity 渲染器渲染函数
     *   @param filePath :    .vm文件路径
     *   @param httpRequst : http request对象
     *   @return :  Promise
     */
    render ( filePath,httpRequst ) {
        let _this = this;

        return new Promise( function ( resolve,reject ) {
            if ( !path.isAbsolute(filePath) ) {
                reject();
            }

            let fileInfo = {
                'dirpath' : path.dirname(filePath),
                'filename' : path.basename(filePath)
            };
            fileInfo = JSON.stringify( fileInfo );

            httpRequst = JSON.stringify( httpRequst );

            let cmd = null;

            cmd = spawn( 'java',['-jar',jarFilePath,'velocity',fileInfo,httpRequst] );
            let stdout = cmd.stdout,
                stderr = cmd.stderr;

            let outDataStr = '',errDataStr = '';
            stdout.on('data',function (chunk) {
                outDataStr += chunk.toString();
            }).on('end',function () {
                _this.toHTML( outDataStr,resolve,reject );
            });

            stderr.on('data',function (chunk) {
                errDataStr += chunk.toString();
            }).on('end',function () {
                //_this.toHTML( errDataStr,resolve,reject );
                console.log(`VelocityRender.js stderr : ${errDataStr}`);
            });
        } );
    }

    toHTML ( dataStr,resolve,reject ) {
        resolve( dataStr );
    }
}


module.exports = VelocityRender;
