'use strict'
/**
 * Created by gaoyong on 16/5/26.
 */
const fs = require('fs');
const path = require('path');

const appConfigObj = require('../../config/app.config.js');

let configFilePath = '';  // 保存服务器配置项的文件的路径
let appStartupObj = {                // 服务器配置项对象
    port: appConfigObj.PORT_DEFAULT,
    webPort: appConfigObj.WEB_PORT_DEFAULT,
    configHttpPath: appConfigObj.CONFIG_HTTP_PATH
};


const appManager = {
    init: function () {
        try {
            configFilePath = this.getSaveConfigFilePath();

            if ( !fs.existsSync( configFilePath ) ) {
                this._writeAppStartup();

                return true;
            }        
        } catch (e) {
            console.log(`appManager init error: ${e}.`);
            return false;
        }
    },
    _readAppStartup: function () {
        try {
            let ret = require(configFilePath);
            delete require.cache[''+configFilePath];
            if ( ret && typeof ret === 'object' ) {
                appStartupObj = ret;
                return appStartupObj;
            }
        } catch (e) {
            console.log(`util.appManager.js readApp : ${e}`);
        }
        return false;
    },
    _writeAppStartup: function () {
        if ( appStartupObj && typeof appStartupObj === 'object' ) {
            let appStartupJSON = JSON.stringify(appStartupObj);
            try {
                fs.writeFileSync(configFilePath, appStartupJSON, 'utf8');
                return true;
            } catch (e) {
                console.log(`util.appManager.js writeAppStartup : ${e}`);
                return false;
            }
        }
    },

    getMinPort : function () {
        readAppStartup();

        return appConfigObj.MIN_PORT;
    },
    getDefaultPort : function () {
        readAppStartup();

        return appStartupObj.port;
    },
    /**
    *  得到http服务器端口号
    */
    getServicePort : function () {
        return appStartupObj.port;      
    },
    /**
    *   得到http服务器域名
    */
    getHttpDomain : function () {
        return appStartupObj.domain;
    },
    savePorts : function ( ports ) {
        let port = ports.port,webPort = ports.webPort;

        //读取上次的配置信息
        this._readAppStartup();

        if ( port > 2000 && port < 60000 ) {
            appStartupObj.port = port;
        } 
        if ( webPort > 2000 && webPort < 60000 ) {
            appStartupObj.webPort = webPort;
        }
       
        // 保存配置信息
        this._writeAppStartup();

        return appStartupObj;
    },

    /*
    *  得到一个fn-server有写权限的用来保存服务器配置项的文件的路径
    */
    getSaveConfigFilePath : function () {
        let tmpPath = process.env.APPDATA || (process.platform == 'darwin' ? 
                path.join(process.env.HOME, 'Library/Preferences') : '/var/local');
        if ( !fs.existsSync(tmpPath) ) {
            fs.mkdirSync( tmpPath );
        }
        
        let fileName = '__fnServer.json';
        let filePath = path.join( tmpPath,fileName );

        return filePath;
    },
    /**
    *  得到系统配置
    */
    getSystemConfig : function () {
        this._readAppStartup();

        return appStartupObj;
    },
    /**
    *  保存系统配置
    */
    saveSystemConfig: function ( systemConfig ) {
        if ( !systemConfig ) {
            return false;
        }

        let servicePort = systemConfig.port,
                webPort = systemConfig.webPort,
                    configHttpPath = systemConfig.configHttpPath;

        if ( isNaN(servicePort) ) {
            return false;
        }

        if ( servicePort < 2000 || servicePort > 60000 ) {
            return false;
        }   

        if ( isNaN(webPort) ) {
            return false;
        }

        if ( webPort < 2000 || webPort >= 9000 ) {
            return false;
        }

        if ( !configHttpPath || !path.isAbsolute(configHttpPath) ) {
            return false;
        }

        // 校验读取属性
        try {
            fs.accessSync( configHttpPath,fs.R_OK )
            
            appStartupObj = systemConfig;
            return this._writeAppStartup();
        } catch (e) {
            return false;
        }
    },
    /*
    *  得到http请求目录
    */ 
    getHttpPath : function () {
        this._readAppStartup();
        
        return appStartupObj.configHttpPath;
    },
    /**
    *  保存http请求目录
    */
    saveHttpPath : function ( httpPath ) {
        if ( path.isAbsolute(httpPath) ) {
            let stat = fs.statSync( httpPath );
            if ( stat.isDirectory() ) { // 是目录
                appStartupObj.configHttpPath = httpPath;
                this._writeAppStartup();
                return true;
            } else {
                return 'httpPath不是目录.';
            }
        }

        return false;
    }
};


appManager.init();
module.exports = appManager;

