'use strict'
/**
*  fn-server HttpUtil工具
*	@author: fe-gaoyong
*	@date: 2016.06.27
*/
const fs = require('fs');
const path = require('path');
const url = require('url');

const glob = require('glob');

const Render = require('../service/renders/Render.js');
const appManager = require('./appManager.js');
const httpFilesPath = appManager.getHttpPath();
const httpFilePattern = path.join( httpFilesPath,'./**/*.fns.js');

class HttpUtil {
	constructor () {

	}

	/**
    *  加载系统落地页
    *   @return Promise
    */
    reloadLandPages () {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._reloadLandPages(resolve,reject);
            } catch (e) {
                reject(e);
            }
        } );
    }
    _reloadLandPages (resolve,reject) {
        if ( !path.isAbsolute(httpFilesPath) || 
                        !fs.existsSync(httpFilesPath) ) {
            reject(`httpFilesPath isnot absolute or not exist.`);
            return false;
        }

        let stat = fs.statSync(httpFilesPath);
        if ( !stat.isDirectory() ) {
            reject(`httpFilesPath isnot directory.`);
            return false;
        }

        /**
        *   扫描http目录下的所有配置文件
        */
        glob( httpFilePattern,{nodir:true},function (error,files) {
            if ( error ) {
                reject(error);
                return false;
            }

            /**
            *   动态加载配置信息脚本文件
            *    将其中的落地页信息取出放到 pathObjArrs 数组中
            */
            let tmpPageProjects = [];  // 临时数组，供存放未过滤的工程
            files.forEach( function (file) {
                try {
                    let tmp = null;
                    tmp = require(file);
                    
                    /*
                    *  核心代码:   
                    *     从node缓存中删除加载的js文件,以便下载require时重新加载js.
                    */
                    delete require.cache[''+file];
                        
                    if ( tmp && Array.isArray(tmp) ) {
                        tmpPageProjects = tmpPageProjects.concat( tmp );
                    }
                } catch (e) {
                    console.log(`pathManager reload file error,error file: ${file},error: ${e}`);
                }
            } );

            /**
            *   过滤掉非落地页
            */
            let pagesPathInfo = [];
            for ( let project of tmpPageProjects ) {
                // 工程名
                const projectName = project.projectName || '';
                // 基础路径
                let baseFilePath = project.baseFilePath || '';


                if ( project.routes && Array.isArray(project.routes) ) {
                    for (let route of project.routes) {
                        // 判断配置项类型是落地页且配置了落地页的路径
                        if ( Render.isValidatePageSetting(route) ) {
                            let abFilePath = '';
                            // 拼接html模版文件的绝对路径
                            if ( !path.isAbsolute(route.filePath) ) {
                                if ( baseFilePath ) {
                                    abFilePath = path.join(baseFilePath,route.filePath);
                                } else {
                                    abFilePath = route.filePath;
                                }
                            } else {
                                abFilePath = route.filePath;
                            }
                            
                            // 过滤出合法的模版配置信息
                            if ( path.isAbsolute(abFilePath) ) {
                                route.filePath = abFilePath;
                                route.projectName = projectName;
                                pagesPathInfo.push(route);
                            } else {
                                console.log(`${route.filePath} is error!`);
                            }
                        }
                    }
                }
            }

            resolve(pagesPathInfo);
        } );
    }


    /**
    *  加载ajax配置项
    *   @return Promise
    */
    reloadAjaxDatas () {
        const _this = this;

        return new Promise( function (resolve,reject) {
            try {
                _this._reloadAjaxDatas(resolve,reject);
            } catch (e) {
                reject(e);
            }
        } );
    }
    _reloadAjaxDatas (resolve,reject) {
        if ( !path.isAbsolute(httpFilesPath) || 
                        !fs.existsSync(httpFilesPath) ) {
            reject(`httpFilesPath isnot absolute or not exist.`);
            return false;
        }

        let stat = fs.statSync(httpFilesPath);
        if ( !stat.isDirectory() ) {
            reject(`httpFilesPath isnot directory.`);
            return false;
        }
        
        /**
        *   扫描http目录下的所有配置文件
        */
        glob( httpFilePattern,{nodir:true},function (error,files) {
            if ( error ) {
                reject(error);
                return false;
            }

            /**
            *   动态加载配置信息脚本文件
            *    将其中的ajax配置信息取出放到 pathObjArrs 数组中
            */
            let tmpAjaxProjects = [];  // 临时数组，供存放未过滤的工程
            files.forEach( function (file) {
                try {
                    let tmp = null;
                    tmp = require(file);
                    
                    /*
                    *  核心代码:   
                    *     从node缓存中删除加载的js文件,以便下载require时重新加载js.
                    */
                    delete require.cache[''+file];
                        
                    if ( tmp && Array.isArray(tmp) ) {
                        tmpAjaxProjects = tmpAjaxProjects.concat( tmp );
                    }
                } catch (e) {
                    console.log(`pathManager reload file error,error file: ${file},error: ${e}`);
                }
            } );

            /**
            *   过滤掉非落地页
            */
            const ajaxsPathInfo = [];
            for ( let project of tmpAjaxProjects ) {
                // 工程名
                const projectName = project.projectName || '';

                if ( project.routes && Array.isArray(project.routes) ) {
                    for (let route of project.routes) {
                        // 判断配置项类型是ajax请求类型 目前仅支持 get/post 
                        if ( Render.isValidAjaxSetting(route) ) {
                            /**
                            *   过滤出合法的模版配置信息
                            *     注意ajax配置中uri需要是只包含路径部分,因为fn-server服务器
                            *     必需监听9000以上端口,所以前端开发人员写代码时,当写到ajax请求 
                            ＊    地址的时候必须像如下这样写： 
                            ＊        '/getsomthing'    '/activity/apiRegister'
                            *     考虑到ajax get/post请求无法跨域,所以此处fn-server服务器
                            ＊     的这种假设是符合现实开发情况的.
                            *
                            *       !!! 以上的假设不能满足最新的xhr2规范，在xhr2中，服务器和
                            *       浏览器都可以设置来让ajax具备跨域的能力
                            ＊        
                            */
                            route.projectName = projectName;

                            ajaxsPathInfo.push(route);
                        } else {
                            console.log(`ajax setting info error : ${route.uri}`);    
                        }
                    }
                }
            }

            resolve(ajaxsPathInfo);
        } );
    }



    /**
    *  将fn-server http服务的端口号加到给定的uri上
    *   @param uri
    */
    addPort2URI (uri) {
        if ( !uri || typeof uri != 'string' || uri.length <= 0 ) {
            return false;
        }

        const port = appManager.getServicePort();

        let uriObj = url.parse( uri );

        /**
        *   干掉host href属性 让url模块将fn-server http端口号放入到retURI中
        */
        delete uriObj.host;
        delete uriObj.href;
        
        uriObj.protocol = 'http:';
        uriObj.port = port;
        let retURI = url.format( uriObj );
        return retURI;
    }


    /**
    *  根据nodejs http request对象拼接出 fn-server服务器规定的 uri
    *    这其中核心的业务逻辑是过滤出 originalUrl 的端口号
    *     另外现在 koa.js中的originalUrl只有http path部分,所以需要
    *     根据request对象中的 headers.host 字段拿到域名和端口
    *   @param uri
    */
    setNoPortURI (req) {
        if ( !req || typeof req != 'object' ||
                !req.headers || typeof req.headers != 'object' ||
                    !req.headers.host || typeof req.headers.host != 'string' ) {
            return false;
        }
        let host = req.headers.host.split(':')[0];
        let retURI = 'http://' + host + req.url;
        
        return retURI;
    }
    

}

const httpUtil = new HttpUtil();

module.exports = httpUtil;


