/**
*  业务逻辑
*   @author: fe-gaoyong
*   @date: 2016.06.30
*/
define(['jquery',"bootstrap","util"],function($,bootstrap,util) {
	
	/**
	*  解锁系统配置
	*/
	function unlockConfig () {
		$('#update').removeAttr('disabled');
		$('#servicePort').removeAttr('disabled');
		$('#webPort').removeAttr('disabled');
		$('#httpPath').removeAttr('disabled');
		//$('#httpDomain').removeAttr('disabled');
	}

	/**
	*  读取系统设置
	*/
	function getSystemSetting () {
		$.ajax( {
			url: '/getSystemSetting',
			method: 'get',
			dataType: 'json',
			success: function (data) {
				if ( typeof data === 'object' ) {
					showSystemSetting(data);
				} else {
					util.showAlert( '服务器返回空!' );
				}
			},
			error: function (xhr,err) {
				util.showAlert( err );
			} 
		} );
	}
	/**
	*  显示系统配置
	*/
	function showSystemSetting (data) {
		unlockConfig();

		if ( !isNaN( data.port ) ) {
			$('#servicePort').val( data.port );
		}
		if ( !isNaN( data.webPort ) ) {
			$('#webPort').val( data.webPort );
		}
		if ( data.configHttpPath ) {
			var configHttpPath = decodeURIComponent( data.configHttpPath );
			$('#httpPath').val( configHttpPath );
		}

		// if ( data.domain ) {
		// 	$('#httpDomain').val( data.domain );
		// }
	}

	function showSaveSystemSetting(data) {
		if ( data['err_code'] === 0 ) {
			util.showAlert('保存成功');
		} else {
			util.showAlert('保存系统配置报错!');
		}
	}
	

	/**
	*  保存系统设置
	*/
	function saveSystemConfig () {
		var servicePort = $('#servicePort').val(),
				webPort = $('#webPort').val(),
					httpPath = $('#httpPath').val(),
						httpDomain = $('#httpDomain').val();

		servicePort = $.trim( servicePort );
		webPort = $.trim( webPort );		
		httpPath = $.trim( httpPath );
		httpDomain = $.trim( httpDomain );

		if ( isNaN(servicePort) ) {
			util.showAlert( '工作端口号必须是数字!' );
			return false;
		}

		if ( isNaN(webPort) ) {
			util.showAlert( '配置服务端口号必须是数字!' );
			return false;
		}

		if ( !httpPath ) {
			util.showAlert( '必须输入http服务配置路径!' );
			return false;
		}

		httpPath = encodeURIComponent(httpPath);
		var postData = {
			'port': servicePort,
			'webPort': webPort,
			'configHttpPath': httpPath
		};

		var _url = '/saveSystemSetting?servicePort=' + servicePort + 
						'&webPort=' + webPort + '&configHttpPath=' + 
								httpPath; //+ '&httpDomain=' + httpDomain;

		$.ajax( {
			url: _url,
			method: 'get',
			dataType: 'json',
			success: function (data) {
				if ( typeof data === 'object' ) {
					showSaveSystemSetting(data);
				} else {
					util.showAlert( '服务器返回空!' );
				}
			},
			error: function (xhr,err) {
				util.showAlert( err );
			} 
		} );
	}


	$('#update').click( function () {
		saveSystemConfig();
	} );

	function init () {
		getSystemSetting();
	}

	setTimeout( function () {
		init();
	},500 );

});

